﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace KhMsgBox
{
    static class Program
    {
        static string versionstring = "2.2019-01-26";
        static string TitleArg = "KhMsgBox";  // デフォールトの、メッセージボックスタイトル
        static string ButtontypeArg = "ok";   // デフォールトのボタンタイプ
        static bool IsUseNlcode = false;      // \n を改行として処理するか。デフォールトは「しない」。
        static int Nargs;             // 一般引数の個数
        static string[] Args;         // 一般引数を格納した配列
        static string progname = "";   // 実行時プログラムの名前

        // 引数を取るオプションの引数を取得する関数
        public static string getnextarg(string optstr, string[] args, ref int ip)
        {   // Read in the next array element as an argument for optstr.
            ip += 1;
            if (ip >= Nargs)
            {
                throw new Exception(optstr + " needs a string argument");
            }
            return args[ip];
        }

        /// <summary>
        /// アプリケーションのメイン エントリ ポイントです。
        /// </summary>
        [STAThread]
        static int Main(string[] cargs)
        {
            // 実行時のプログラムの名前。C:/home/foo.exe なら foo
            progname =
                System.IO.Path.GetFileNameWithoutExtension(
                    Application.ExecutablePath);

            TitleArg = progname;

            string optstr = "";  // コマンドライン引数の解析に利用します。
            Nargs = cargs.Length;

            // Func<T1,TResulst> は、引数T1型、戻り値TResult型のデリゲー
            // ト型。
            // Func<T1,T2,Tresult> は引数T1型、T2型、戻り値TResult型の
            // デリゲート型。

            // デリゲート isOption をラムダ式を用いて定義
            Func<string, string, bool> isOption =
                (input, optionName) => Regex.IsMatch(input,
                "^(/|-)" + optionName + "$");
            Func<string, bool> is_title = (input) => isOption(input, "title");
            Func<string, bool> is_buttontype = (input) => isOption(input, "buttontype");
            Func<string, bool> is_usenlcode = (input) => isOption(input, "usenlcode");

            int ip = 0;

            try
            {
                for (; ip < Nargs; ++ip)
                {
                    optstr = cargs[ip];
                    if (Regex.IsMatch(optstr, "^(/|-)"))
                    {
                        string loptstr = optstr.ToLower();
                        if (is_title(loptstr))
                        {
                            // -title は値を取るオプション
                            TitleArg = getnextarg(optstr, cargs, ref ip);
                            // FooArg はオプション /foo の値
                        }
                        else if (is_buttontype(loptstr))
                        {
                            // -buttontype は値を取るオプション
                            ButtontypeArg = getnextarg(optstr, cargs, ref ip);
                        }
                        else if (is_usenlcode(loptstr))
                        {
                            // -usenlcode は値を取らないオプション
                            // \n は改行として処理
                            IsUseNlcode = true;
                        }
                        else
                        {
                            throw new Exception(optstr + ": unrecognised option");
                        }
                    }
                    else
                        break;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, progname);
                Application.Exit();
                System.Environment.Exit(-1);
            }

            MessageBoxButtons messageboxButton;

            ButtontypeArg = ButtontypeArg.ToLower();

            switch (ButtontypeArg)
            {
                case "abortretryignore":
                    messageboxButton = MessageBoxButtons.AbortRetryIgnore;
                    break;
                case "ok":
                    messageboxButton = MessageBoxButtons.OK;
                    break;
                case "okcancel":
                    messageboxButton = MessageBoxButtons.OKCancel;
                    break;
                case "yesno":
                    messageboxButton = MessageBoxButtons.YesNo;
                    break;
                case "yesnocancel":
                    messageboxButton = MessageBoxButtons.YesNoCancel;
                    break;
                case "retrycancel":
                    messageboxButton = MessageBoxButtons.RetryCancel;
                    break;
                default:
                    messageboxButton = MessageBoxButtons.OK;
                    break;           
            }

            // 後は、一般引数
            Nargs = Nargs - ip;
            if (Nargs > 0)
                Args = new string[Nargs];

            for (int i = 0; i < Nargs; i++)
            {
                Args[i] = cargs[i + ip];
            }

            string message = "";

            if (Nargs > 0)
            {
                for (int i = 0; i < Nargs; i++)
                {
                    if (Args[i] == "NewLine")
                    {
                        message += Environment.NewLine;
                    }
                    else if (Args[i] == "Cr")
                    {
                        message += Environment.NewLine;
                    }
                    else
                    {
                        message += Args[i];
                    }
                }
                if (IsUseNlcode)
                {
                    message = message.Replace("\\n", Environment.NewLine);
                }
                
            }
            else
            {
                // メッセージが何も指定されなかった場合は、使用方法に関する説明を
                // message に格納します。
                message = @"
version: versionstring

呼び出し例

  KhMsgBox.exe -Title foo -ButtonType YesNoCancel ^
  ボタンを NewLine クリックしてください。

引数をとるオプション -Title を指定すると、引数文字列をメッセージ・ボックスのタイトル部分に表示します。

デフォールトでは、ボタンの種類は OK ですが、引数をとるオプション -ButtonType オプションを指定して、表示されるボタンの種類 (AbortRetryIgnore, OK, OKCancel, YesNo, YesNoCancel, RetryCancel) を指定できます。

文字列 NewLine または Cr があるとそこで改行されます。呼び出し例では、「ボタンを」で改行し、つぎの行に「クリックしてください。」と表示されます。

-UseNlcode を指定すると、文字列の中に \n があるとそこで改行されます。-UseNlcode を指定しなければ、文字列の中に \n があってもそこで改行されることはありません。

KhMsgBox.exe はユーザがクリックしたボタンがどれかを示す整数値を返します。

返された整数値とボタンの対応は、0->Abort, 1->Cancel, 2->Ignore, 3->No, 4->OK, 5->Retry, 6->Yes です。

引数を指定せずに KhMsgBox.exe を起動するとこの使用説明文が表示されます。

";
                message = message.Replace("versionstring", versionstring);
                message = message.Replace("KhMsgBox", progname);
            }

            int iret = 4;
            DialogResult ret;
            ret = MessageBox.Show(message, TitleArg, messageboxButton);

            switch (ret)
            {
                case DialogResult.Abort:
                    iret = 0;
                    break;
                case DialogResult.Cancel:
                    iret = 1;
                    break;
                case DialogResult.Ignore:
                    iret = 2;
                    break;
                case DialogResult.No:
                    iret = 3;
                    break;
                case DialogResult.OK:
                    iret = 4;
                    break;
                case DialogResult.Retry:
                    iret = 5;
                    break;
                case DialogResult.Yes:
                    iret = 6;
                    break;
            }

            return iret;

            /*
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new Form1());
            */
        }
    }
}
